

# file utility used to find out mime type of a file
BINARY_FILE = "/usr/bin/file"

# Provided by ImageMagick package.
# Used for resizing images.
BINARY_CONVERT = "/usr/bin/convert"

# Provided by Poppler Utils.
# Used to extract images from PDF file.
BINARY_PDFTOPPM = "/usr/bin/pdftoppm"

# Provided by Poppler Utils.
# used to get page count in PDF file
BINARY_PDFINFO = "/usr/bin/pdfinfo"

# Provided by ImageMagick package.
# Used to get number of pages in TIFF file.
BINARY_IDENTIFY = "/usr/bin/identify"

# Provided by tesseract package.
# Used to extract text from images/PDF files.
BINARY_OCR = "/usr/bin/tesseract"

# Provided by stapler
# Used to edit PDF documents
BINARY_STAPLER = "~/.local/bin/stapler"
